'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (config) {
  return function (css) {
    const unwrappedConfig = config();

    css.walkAtRules('variants', atRule => {
      const variants = _postcss2.default.list.comma(atRule.params);

      if (variants.includes('responsive')) {
        const responsiveParent = _postcss2.default.atRule({ name: 'responsive' });
        atRule.before(responsiveParent);
        responsiveParent.append(atRule);
      }

      atRule.before(atRule.clone().nodes);

      _lodash2.default.forEach(['focus', 'hover', 'group-hover'], variant => {
        if (variants.includes(variant)) {
          variantGenerators[variant](atRule, unwrappedConfig);
        }
      });

      atRule.remove();
    });
  };
};

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _buildClassVariant = require('../util/buildClassVariant');

var _buildClassVariant2 = _interopRequireDefault(_buildClassVariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const variantGenerators = {
  hover: (container, config) => {
    const cloned = container.clone();

    cloned.walkRules(rule => {
      rule.selector = `${(0, _buildClassVariant2.default)(rule.selector, 'hover', config.options.separator)}:hover`;
    });

    container.before(cloned.nodes);
  },
  focus: (container, config) => {
    const cloned = container.clone();

    cloned.walkRules(rule => {
      rule.selector = `${(0, _buildClassVariant2.default)(rule.selector, 'focus', config.options.separator)}:focus`;
    });

    container.before(cloned.nodes);
  },
  'group-hover': (container, config) => {
    const cloned = container.clone();

    cloned.walkRules(rule => {
      // prettier-ignore
      rule.selector = `.group:hover ${(0, _buildClassVariant2.default)(rule.selector, 'group-hover', config.options.separator)}`;
    });

    container.before(cloned.nodes);
  }
};